#ifndef HEPVis_SbHatching_h
#define HEPVis_SbHatching_h

/**
 *  Sohatching.h
 *  
 *
 *  Created by Laurent Garnier on Fri Jan 30 2004.
 * 
 * This class builds a list of hatching. 
 * It does nothing more than call
 * SbHatch and store results in a vector of SbVec3
 *
 */

#include <Inventor/SbLinear.h>
#include <vector>

#include <HEPVis/SbHatch.h>

class SbHatching : public SbHatch { 
public:
  SbHatching();
  virtual ~SbHatching();
  
  /** create a new polyline hatched and append result in a point vector
  * Store nothing if the hatch can't be draw
  * return TRUE if the polyline had been computed, false otherwise
  */
  int addPolyline(SbVec3f* Coord,int number);
  
  /** Get the list of points of all the hatch already computed
   * You should also get the hatchNumVerticeList to draw the hatch
   * For a hatch with a 0 thickness (by default), the hatchNumberVericeList will be a full of '2'
   */
  const std::vector<SbVec3f>& getHatchPointsList() const;

  /** Get the number of vertices list.
   * For a hatch with a 0 thickness (by default), the hatchNumberVericeList will be a full of '2'
   */
  const std::vector<unsigned int>& getHatchNumVerticesList() const;
 
  /** Get the minimum corner of Bounding box for all hatch already computed */
  const SbVec3f& getCornerMinBB();
  
  /** Get the maximum corner of Bounding box for all hatch already computed */
  const SbVec3f& getCornerMaxBB();
  
  /** Get the number of points in the list
   * <font color="red">Be careful with this function because it can return a set of non convex
   * polygone if you have a non convex polygone at beginning !So, when you want
   * to draw it, you have to use a tesselisation algorithm first</font>
   */
  const unsigned int getNumPointsList() const;

  /** Get the number of vertices */
  const unsigned int getNumVertices() const;

private:
  std::vector<SbVec3f> fHatchPointsList;
  std::vector<unsigned int> fHatchNumVerticesList;
  SbVec3f fCornerMinBB;
  SbVec3f fCornerMaxBB;
};

#endif
