#ifndef Lib_Session_h
#define Lib_Session_h

// Inheritance :
#include <Slash/Core/ISession.h>

// Fields :
#include <Lib/Parameters.h>
#include <Lib/StreamBuffer.h>

namespace Lib {

class Session : public virtual Slash::Core::ISession {
public: //Slash::Core::ISession
  virtual void* cast(const std::string&) const;

  virtual void addManager(Slash::Core::IManager*);
  virtual void removeManager(Slash::Core::IManager*);
  virtual Slash::Core::IManager* findManager(const std::string&,bool = true) const;
  virtual void destroyManager(const std::string&);
  virtual unsigned int numberOfManagers() const;
  virtual const Slash::Core::IManager* manager(unsigned int) const;
  virtual std::vector<std::string> managerNames() const;

  virtual Slash::Core::IWriter& printer();
  virtual std::ostream& cout();
  virtual void setVerboseLevel(int);
  virtual int verboseLevel() const;

  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool removeParameter(const std::string&);
  virtual std::vector<std::string> availableParameters() const;

  virtual std::vector< std::pair<std::string,std::string> > arguments() const;

  virtual bool load(const std::string&,const std::string&);
  virtual void redirectOutput(bool);
  virtual bool flush();
public:
  Session(Slash::Core::IWriter&,bool balance = false,bool check_class = false);
  virtual ~Session();
protected:
  Session(const Session&);
private:
  Session& operator=(const Session&);
public:
  void setArguments(const std::vector< std::pair<std::string,std::string> >&);
protected:
  bool rm_managers();
private:
  Slash::Core::IWriter& fPrinter;
  Lib::StreamBuffer fStreamBuffer;
  std::ostream f_cout;
  bool fBalance;
protected:
  int fVerboseLevel;
private:
  std::vector<Slash::Core::IManager*> fManagers;
  std::vector< std::pair<std::string,std::string> > fArguments;
  Lib::Parameters<std::string,std::string> fParameters;
};

}

#endif
