#ifndef Slash_Tools_InventorAccessor_h
#define Slash_Tools_InventorAccessor_h

// Inheritance :
#include <Slash/Data/IAccessor.h>
#include <Slash/Data/IVisualizer.h>

#include <Slash/Core/ISession.h>
#include <Slash/Data/IProperty.h>
#include <Slash/UI/IUI.h>
#include <Slash/UI/IWidget.h>
#include <Slash/UI/ISoViewer.h>
#include <Slash/Tools/Managers.h>
#include <Slash/Tools/UI.h>
#include <Slash/Tools/args.h>
#include <Slash/Tools/Inventor.h>

#include <HEPVis/misc/SoGC.h>
#include <HEPVis/nodekits/SoRegion.h>

namespace Slash {

class InventorAccessor 
:public virtual Slash::Data::IAccessor
,public virtual Slash::Data::IVisualizer {
public: //Slash::Data::IAccessor.
  virtual unsigned int numberOfProperties() const {return fProps.size();}
  virtual const Slash::Data::IProperty* property(unsigned int aIndex) const {
    if(aIndex>=fProps.size()) return 0;
    return fProps[aIndex];
  }
  virtual Slash::Data::IIterator* iterator() {return 0;}
  virtual bool setName(const std::string&) {return false;}
  virtual void setIterator(Slash::Data::IIterator*){}
  virtual void setIteratorArguments(const std::vector<std::string>& aArgs){fIteratorArgs = aArgs;}
  virtual void destroy(Slash::Data::IAccessor::Data,void*){}
  virtual void set(Slash::Data::IAccessor::Data,const std::string&,
                   const std::string&,void*){}
  virtual void calculate(Slash::Data::IAccessor::Data,const std::string&,
                         const std::vector<Slash::Core::IValue*>&,void*){}
  virtual void setExecuteArguments(const std::vector<std::string>& aArgs){fExecuteArgs = aArgs;}
public: //Slash::Data::IAccessor.
  virtual void* cast(const std::string& aClass) const {
    if(aClass=="Slash::InventorAccessor") {
      return (void*)static_cast<const Slash::InventorAccessor*>(this);
    } else if(aClass=="Slash::Data::IVisualizer") {
      return (void*)static_cast<const Slash::Data::IVisualizer*>(this);
    } else if(aClass=="Slash::Data::IAccessor") {
      return (void*)static_cast<const Slash::Data::IAccessor*>(this);
    } else {
      return 0;
    }
  }
public: //Slash::Data::IVisualizer.
  virtual void beginVisualize() {
    fSoGC.clear(); //NOTE : see endVisualize().
    fSoRegion = 0;
    fWhere.clear();
    fStyle.clear();
    Slash::UI::IUI* ui = Slash::find_UI(fSession);
    if(!ui) {
      fSoGC.setDefaultStyleCache();
    } else {
     {const std::vector<std::string>& eargs = fExecuteArgs;
      if(eargs.size()) {
        Slash::args args(eargs);
        args.find("where",fWhere);
      }}
      fSoRegion = Slash::find_SoRegion(*ui,fWhere);
      if(!fSoRegion) {
        fSoGC.setDefaultStyleCache();
      } else {
        fSoGC.setStyleCache(fSoRegion->styleCache());
        fillSoGC(name()); //By default get a style with same name as the type.
      }
    }
   {const std::vector<std::string>& eargs = fExecuteArgs;
    if(eargs.size()) {
      Slash::args args(eargs);
      args.find("style",fStyle);
    }}
  }
  //virtual void visualize(Slash::Data::IAccessor::Data,void*){
  //  Still pure virtual.
  //}
  virtual void endVisualize() {
    //FIXME : if removing the fSoGC.clear in beginVisualise
    // we should have to revisit all derived class having a endVisualize.
    // to call InventorAccessor::endVisualize()
    fSoGC.clear(); 
  }
public:
  InventorAccessor(Slash::Core::ISession& aSession,
                   const std::string& aStyleFile = "")
  :fSession(aSession)
  ,fStyleManager(0)
  ,fSoRegion(0){
    fStyleManager = Slash::styleManager(fSession);
    if(fStyleManager && (aStyleFile.size()) ) {
      fStyleManager->loadFile(aStyleFile);
    }
  }
  virtual ~InventorAccessor() {
    clearProperties();
  }
private:
  InventorAccessor(const InventorAccessor& aFrom)
  :fSession(aFrom.fSession),fStyleManager(0),fSoRegion(0){}
  InventorAccessor& operator=(const InventorAccessor&){return *this;}
public:
  bool loadStyleFile(const std::string& aStyleFile) {
    if(!fStyleManager) return false;
    return fStyleManager->loadFile(aStyleFile);
  }
  bool isStyle(const std::string& aStyle) const {
    if(!fStyleManager) return false;
    return fStyleManager->isStyle(aStyle);
  }
protected: //BaseAccessor
  void addProperty(Slash::Data::IProperty* aProperty) { //it takes ownership
    fProps.push_back(aProperty);
  }
  void clearProperties(){
    std::vector<Slash::Data::IProperty*>::iterator it;
    for(it=fProps.begin();it!=fProps.end();++it) delete *it;
    fProps.clear();
  }
protected:
  void fillSoGC(const std::string& aStyle = "") {
    fSoGC.reset();

    // Style priority : we look first for an "exec style" coming
    // as an argument to the data_visualize. If no found then we
    // look for aStyle. And anyway, if not "none", 
    // session modeling.[color,...] overrides file style.

   {std::string value;
    if(fStyleManager && 
         fStyleManager->findStyle(fStyle.size()?fStyle:aStyle,value)) { 
      fSoGC.setFromString(SbString(value.c_str()));
    }}

    // If not "none", session modeling.[color,...] overrides file style.

    std::string sstyle;
    std::string value;
    fSession.parameterValue("modeling.color",value);
    if((value.size())&&(value!="none")) sstyle += "\ncolor "+value;

    fSession.parameterValue("modeling.highlightColor",value);
    if((value.size())&&(value!="none")) sstyle += "\nhighlightColor "+value;

    fSession.parameterValue("modeling.transparency",value);
    if((value.size())&&(value!="none")) sstyle += "\ntransparency "+value;
  
    fSession.parameterValue("modeling.lineStyle",value); // Deprecated :
    if((value.size())&&(value!="none")) sstyle += "\nlineStyle "+value;

    fSession.parameterValue("modeling.linePattern",value);
    if((value.size())&&(value!="none")) sstyle += "\nlinePattern "+value;

    fSession.parameterValue("modeling.lineWidth",value);
    if((value.size())&&(value!="none")) sstyle += "\nlineWidth "+value;
  
    //Polyhedron :
    fSession.parameterValue("modeling.projection",value);
    if((value.size())&&(value!="none")) sstyle += "\nprojection "+value;

    fSession.parameterValue("modeling.cut",value);
    if((value.size())&&(value!="none")) sstyle += "\ncut "+value;

    fSession.parameterValue("modeling.rotationSteps",value);
    if((value.size())&&(value!="none")) sstyle += "\nrotationSteps "+value;

    fSession.parameterValue("modeling.modeling",value);
    if((value.size())&&(value!="none")) sstyle += "\nmodeling "+value;
  
    //Backward compatibility.
    fSession.parameterValue("modeling.type",value);
    if((value.size())&&(value!="none")) {
      if(value=="text") {
        sstyle += "\nmodeling texts";
      } else {
        sstyle += "\nmodeling "+value;
      }
    }
  
    fSoGC.setFromString(SbString(sstyle.c_str()));

    fSession.parameterValue("modeling.localSetup",value);
    if((value.size())&&(value!="none")) {
      bool b;
      if(Slash::to(value,b)) {
        fSoGC.setLocalSetup(b?TRUE:FALSE);
      }
    }


  }
  SoGC& soGC() { return fSoGC;}
  SoRegion* soRegion() const { return fSoRegion;}
  std::string where() const { return fWhere;}
private: //BaseAccessor
  std::vector<Slash::Data::IProperty*> fProps;
  std::vector<std::string> fIteratorArgs;
  std::vector<std::string> fExecuteArgs;
protected:
  Slash::Core::ISession& fSession;
  Slash::UI::IStyleManager* fStyleManager;
  SoRegion* fSoRegion;
  SoGC fSoGC;
  std::string fWhere;
  std::string fStyle;
};

}

#endif
