#ifndef Slash_file_h
#define Slash_file_h

#include <string>
#include <cstdio>
#include <cstring>
#include <vector>

namespace Slash {
namespace file {

inline bool exists(const std::string& a_string) {
  FILE* file = ::fopen(a_string.c_str(),"rb");
  if(!file) return false;
  ::fclose(file);
  return true;
}

inline bool write(const std::string& a_file,const std::vector<std::string>& a_text){
  FILE* file = ::fopen(a_file.c_str(),"wb");
  if(!file) return false;
  std::vector<std::string>::const_iterator it;
  for(it=a_text.begin();it!=a_text.end();++it) {
    if(::fprintf(file,"%s\n",(*it).c_str())<0) {
      ::fclose(file);
      return false;
    }
  }
  ::fclose(file);
  return true;
}

inline bool read(const std::string& a_file,std::vector<std::string>& a_text){
  a_text.clear();
  //FIXME : should read without a limited buffer.
  FILE* file = ::fopen(a_file.c_str(),"rb");
  if(!file) return false;
  unsigned int BUFSIZE = 65536;
  char* buffer = new char[BUFSIZE+1];
  if(!buffer) {::fclose(file);return false;}
  while(true) {
    if(::fgets(buffer,BUFSIZE,(FILE*)file)==NULL) break; // EOF.
    int l = ::strlen(buffer);
    //  On Windows, editors when saving binary files,
    // put \r\n at place of \n ; we then look for \r\n.
    if( (l>=2) && (buffer[l-2]=='\r') && (buffer[l-1]=='\n') ) {
      buffer[l-2] = '\0';
    } else if( (l>=1) && (buffer[l-1]=='\n') ) {
      buffer[l-1] = '\0';
    }
    a_text.push_back(buffer);
  }
  delete [] buffer;
  ::fclose(file);
  return true;
}

inline bool copy(const std::string& a_from,const std::string& a_to){
  if(a_to==a_from) return true; //done
  std::vector<std::string> text;
  if(!Slash::file::read(a_from,text)) return false;
  return Slash::file::write(a_to,text);
}


inline bool found(const std::string& a_file,const std::string& a_what,std::vector<std::string>& a_found){
  a_found.clear();
  std::vector<std::string> text;
  if(!Slash::file::read(a_file,text)) return false;
  std::vector<std::string>::const_iterator it;
  for(it=text.begin();it!=text.end();++it) {
    if((*it).find(a_what)!=std::string::npos) {
      a_found.push_back(*it);
    }
  }
  return true;
}

inline bool remove(const std::string& a_file) {
  return (::remove(a_file.c_str()) ==0 ? true : false);
}

inline bool rename(const std::string& a_from,const std::string& a_to) {
  //NOTE : a_from must not be a path !
  //       Darwin is ok with a path but not Linux !
  //       For example : 
  //         ::rename("/tmp/tmp01"."x");
  //       return -1 on Linux.
  //       To do the upper then someone must use move. 
  //       But there is no move in the standard lib C !
  return (::rename(a_from.c_str(),a_to.c_str()) == 0 ? true : false);
}

/*
inline bool replace(const std::string& a_file,const std::string& a_old,const std::string& a_new,std::vector<std::string>& a_changes){
  a_changes.clear();
  std::vector<std::string> text;
  if(!read(a_file,text)) return false;
  // Replace :
  bool touched = false;
 {std::vector<std::string>::const_iterator it;
  for(it=text.begin();it!=text.end();++it) {
    std::string old = *it;
    if(smanip::replace(*it,a_old,a_new)) {
      a_changes.push_back(old);
      a_changes.push_back(*it);
      touched = true;
    }
  }}
  if(!touched) return true;
  // Create temporary file :
  //NOTE : have a tmpName in current directory so that the below 
  //       rename works. (Else we should do a move).
  std::string tmp_name;
  if(!tmpname(tmp_name,".","Lib_")) return false;
  FILE* file = ::fopen(tmp_name.c_str(),"wb");
  if(!file) return false;
 {unsigned int textn = text.size();
  for(unsigned index=0;index<textn;index++) {
    if(::fprintf(file,"%s\n",text[index].c_str())<0) {
      ::fclose(file);
      remove(tmp_name);
      return false;
    }
  }}
  ::fclose(file);
  if(!remove(a_file)) return false;
  if(!rename(tmp_name,a_file)) return false;
  return true;
}
*/

}}

#endif
